@@ -12,8 +12,8 @@
 /* by Arnaud Le Hors, lehors@sophia.inria.fr,       */
 /* Copyright 1990-93 GROUPE BULL                    */
 
-/* This is Xteddy version 2.1 as of 2005-05-25.     */
-/* from  Richard Neill <rn214@hermes.cam.ac.uk>     */
+/* This is Xteddy version 2.2 as of 2009-02-25.     */
+/* from  Peter De Wachter <pdewacht@gmail.com>      */
 /* Changes: see ChangeLog                           */
 
 #define DEFAULT_IMAGE_DIR "/usr/share/xteddy"
@@ -28,7 +28,7 @@
 #include <X11/extensions/shape.h>
 #include <X11/cursorfont.h>
 
-#include <Imlib.h>
+#include <Imlib2.h>
 
 #include <ctype.h>
 #include <stdio.h>
@@ -130,17 +130,23 @@ int main(int argc, char **argv)
 {
   /* Display, window and gc manipulation variables */
   Window               win;
+  Pixmap               pm_image, pm_mask;
   XSetWindowAttributes setwinattr;
   unsigned long        valuemask, inputmask;
-  int                  x, y, geomflags, xw, xh;
+  int                  x, y, geomflags;
+  unsigned int         xw, xh;
   unsigned int         border_width = 0;
   unsigned int         display_width, display_height;
   XSizeHints           size_hints;
   XWMHints             wm_hints;
   XClassHint           class_hints;
+#ifdef FORTIFY_SOURCE
+  XTextProperty        windowName, iconName;
+#else
   XTextProperty        windowName;
+#endif
   int                  argnum;
-  int                  use_wm, float_up, allow_quit;
+  int                  use_wm, float_up, allow_quit, cursor_blank;
   XEvent               report;
   char                *display_name = NULL;
   char                 buffer[20];
@@ -158,8 +164,8 @@ int main(int argc, char **argv)
   int                  offs_x, offs_y, new_x, new_y, tmp_x, tmp_y;
   unsigned int         tmp_mask;
 
-  ImlibData           *id;
-  ImlibImage          *im;
+  Imlib_Image         *im;
+  int                  im_width, im_height;
 
   /* Determine program name */
   if ((progname = strrchr(argv[0],'/')) == NULL)
@@ -169,14 +175,15 @@ int main(int argc, char **argv)
   teddy = progname;
 
 
-  /* Option handling: "-wm", "-float", "-noquit", "-geometry" */
-  /* and "-display" are recognized. See manual page for details. */
+  /* Option handling: "-wm", "-float", "-noquit", "-geometry", */
+  /* "-display" and "-nocursor" are recognized. See manual page for details. */
   /* -F<name> ... Other pixmap name */
   /* -F or -f are equivalent. A space may be included if desired after the option (eg "-f xduck") */
   /* -v prints version. -h or --help prints help */
   use_wm = FALSE;
   float_up = FALSE;
   allow_quit = TRUE;
+  cursor_blank = FALSE;
   x = y = 0;
   geomflags = 0;
   for ( argnum = 1; argnum < argc; argnum++ ) {
@@ -190,6 +197,8 @@ int main(int argc, char **argv)
       geomflags = XParseGeometry(argv[++argnum], &x, &y, &xw, &xh);
     if (!strcmp(argv[argnum],"-display"))
       display_name = argv[++argnum];
+    if (!strcmp(argv[argnum],"-nocursor"))
+      cursor_blank = TRUE;
     /* Use -f or -F; -f seems more natural. But -float does not mean "-f loat" ! */
     if ( (!strncmp(argv[argnum],"-F", 2)) || ( (!strncmp(argv[argnum],"-f", 2)) && (strcmp(argv[argnum],"-float")) ) )
       teddy = argv[argnum] + 2;
@@ -232,33 +241,51 @@ int main(int argc, char **argv)
   display_width  = DisplayWidth(display, screen_num);
   display_height = DisplayHeight(display, screen_num);
 
-  if ( !(id=Imlib_init(display)) ) return -1;
+  /* set the maximum number of colors to allocate for 8bpp and less to 128 */  
+  imlib_set_color_usage(128);
+  /* dither for depths < 24bpp */
+  imlib_context_set_dither(1);
+  /* set the display, visual and colormap we are using */  
+  imlib_context_set_display(display);
+  imlib_context_set_visual(DefaultVisual(display, DefaultScreen(display)));
+  imlib_context_set_colormap(DefaultColormap(display, DefaultScreen(display)));
+
   if ( !(file = InitTeddy(teddy)) ) {
     fprintf(stderr, "Can not find any image with name '%s'.\n", teddy);
     return -1;
   }
-  if ( !(im=Imlib_load_image(id, file)) ) {
+  if ( !(im=imlib_load_image_immediately(file)) ) {
     fprintf(stderr, "Most probably, the file '%s' is not a valid image.\n", teddy);
     return -1;
   }
+  imlib_context_set_image(im);
+  im_width = imlib_image_get_width();
+  im_height = imlib_image_get_height();
   /* Set the window position according to user preferences */
   if (geomflags & XNegative)
-    x = display_width - im->rgb_width + x;
+    x = display_width - im_width + x;
   if (geomflags & YNegative)
-    y = display_height - im->rgb_height + y;
+    y = display_height - im_height + y;
   /* Clip against bounds to stay on the screen */
   if (x<0) x=0;
-  if (x > display_width - im->rgb_width) x = display_width - im->rgb_width;
+  if (x > display_width - im_width) x = display_width - im_width;
   if (y<0) y=0;
-  if (y > display_height - im->rgb_height) y = display_height - im->rgb_height;
+  if (y > display_height - im_height) y = display_height - im_height;
 
   /* Create the main window */
   win = XCreateSimpleWindow(display, DefaultRootWindow(display),
-			    x,y,im->rgb_width,im->rgb_height,border_width,
+			    x,y,im_width,im_height,border_width,
 			    BlackPixel(display,screen_num),
 			    WhitePixel(display,screen_num));
   XSelectInput(display,win,StructureNotifyMask);
-  Imlib_apply_image(id,im,win);
+
+  imlib_context_set_drawable(win);
+  imlib_render_pixmaps_for_whole_image(&pm_image, &pm_mask);
+  XSetWindowBackgroundPixmap(display, win, pm_image);
+  XShapeCombineMask(display, win, ShapeBounding, 0, 0, pm_mask, ShapeSet);
+  XFreePixmap(display, pm_image);
+  XFreePixmap(display, pm_mask);
+  imlib_free_image_and_decache();
 
   basewin = win;
 
@@ -266,21 +293,37 @@ int main(int argc, char **argv)
     setwinattr.override_redirect = FALSE;
   else
     setwinattr.override_redirect = TRUE;
-  cursor = XCreateFontCursor(display, XC_heart);
+
+  if (cursor_blank)
+    {
+      Pixmap blank;
+      XColor dummy;
+      const char data[1] = { 0 };
+      blank = XCreateBitmapFromData (display, win, data, 1, 1);
+      if (blank == None)
+        {
+          fprintf (stderr, "error: out of memory.\n");
+          exit (1);
+        }
+      cursor = XCreatePixmapCursor (display, blank, blank, &dummy, &dummy, 0, 0);
+      XFreePixmap (display , blank);
+    }
+  else
+    cursor = XCreateFontCursor(display, XC_heart);
+
   setwinattr.cursor = cursor;
   valuemask = CWOverrideRedirect | CWCursor;
   XChangeWindowAttributes(display, win, valuemask, &setwinattr);
 
   /* Report size hints and other stuff to the window manager */
-  size_hints.min_width  = im->rgb_width;    /* Don't allow any resizing */
-  size_hints.min_height = im->rgb_height;
-  size_hints.max_width  = im->rgb_width;
-  size_hints.max_height = im->rgb_height;
+  size_hints.min_width  = im_width;    /* Don't allow any resizing */
+  size_hints.min_height = im_height;
+  size_hints.max_width  = im_width;
+  size_hints.max_height = im_height;
   size_hints.flags = PPosition | PSize | PMinSize | PMaxSize;
   if (XStringListToTextProperty(&(teddy), 1, &windowName) == 0)
     {
       (void) fprintf(stderr, "%s: structure allocation for windowName failed.\n", progname);
-      Imlib_kill_image(id, im);
       return -1;
     }
   wm_hints.initial_state = NormalState;
@@ -290,8 +333,13 @@ int main(int argc, char **argv)
   class_hints.res_name = progname;
   class_hints.res_class = "Xteddy";
 
+#ifdef FORTIFY_SOURCE
+  XSetWMProperties(display, win, &windowName, &iconName,
+		   argv, argc, &size_hints, &wm_hints, &class_hints);
+#else
   XSetWMProperties(display, win, &windowName, NULL,
 		   argv, argc, &size_hints, &wm_hints, &class_hints);
+#endif
 
   /* Select event types wanted */
   inputmask = ExposureMask | KeyPressMask | ButtonPressMask |
@@ -309,20 +357,6 @@ int main(int argc, char **argv)
       XNextEvent(display, &report);
       switch(report.type)
 	{
-	case Expose:
-	  if (report.xexpose.count != 0)
-	    break;
-	  else
-	    {
-	      /* No drawing needed - the background pixmap */
-	      /* is handled automatically by the X server  */
-	    }
-	  break;
-	case ConfigureNotify:
-	  /* Window has been resized */
-          Imlib_apply_image(id,im,win);
-          XSync(display,False);
-	  break;
 	case ReparentNotify:
 	  /* Window was reparented by the window manager */
 	  if (!use_wm)
@@ -382,7 +416,6 @@ int main(int argc, char **argv)
 	    {
               if (allow_quit)
 		{
-                  Imlib_kill_image(id, im);
 		  XCloseDisplay(display);
 		  return 0;
 		}
